<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\Report\ReportService;

use Zeedhi\Framework\DTO;

class Cus01100 extends \Zeedhi\Framework\Controller\Report { 

    protected $novoCodigo;
    protected $procedureConsumoPlanejado;
    protected $parameterBagColumns;
    protected $servicoPorGrupo;
    protected $qr2Strategy;
    protected $birtStrategy;
    protected $environment;
    protected $crudService;
    protected $connection;

    public function __construct($reportService, $qr2Strategy, $birtStrategy, $novoCodigo, $connection, $servicoPorGrupo, $util, $environment, $crudService){
        $this->qr2Strategy               = $qr2Strategy;
        $this->birtStrategy              = $birtStrategy;
        $this->novoCodigo                = $novoCodigo;
        $this->connection                = $connection;
        $this->servicoPorGrupo           = $servicoPorGrupo;
        $this->reportService             = $reportService;
        $this->util                      = $util;
        $this->environment               = $environment;
        $this->crudService               = $crudService;
    }

    protected $reportMapping = array (
        'CUS01100' => 
        array (
            'strategy'         => 'BIRT',
            'parameterMapping' => 
            array (
                'NRORG'           => 'P_NRORG',
                'DTINICIAL'       => 'P_DTINICIAL',
                'DTMESINIC'       => 'P_DTMESINIC',
                'DTMESFINA'       => 'P_DTMESFINA',
                'IDESTOCAVEIS'    => 'P_IDESTOCAVEIS',
                'IDNESTOCAVEIS'   => 'P_IDNESTOCAVEIS',
                'DTPERIFINA'      => 'P_DTPERIFINA',
                'IDDIFERENCA'     => 'P_IDDIFERENCA',
                'DTFINAL'         => 'P_DTFINAL',
                'CDTIPOCUSTOPLAN' => 'P_NRTIPOCUSTOPLAN',
                'CDTIPOCUSTOREAL' => 'P_NRTIPOCUSTOREAL',
                'DTPERIINIC'      => 'P_DTPERIINIC',
                'NMTIPOCUSTOREAL' => 'P_NMTIPOCUSTOREAL',
                'TPDTCUS'         => 'P_TPDTCUS',
                'LISTASERV'       => 'L_SERVICO',
                'NMTIPOCUSTOPLAN' => 'P_NMTIPOCUSTOPLAN',
                'NRCONSPLAN'      => 'P_NRCONSPLAN',
                'LISTAFILI'       => 'L_FILIAL',
                'CDFILIAL'        => 'P_CDFILIAL',
            ),
        ),
        'CUS01101' => 
        array (
        'strategy' => 'BIRT',
        'parameterMapping' => 
            array (
                'NRORG'           => 'P_NRORG',
                'DTINICIAL'       => 'P_DTINICIAL',
                'DTMESINIC'       => 'P_DTMESINIC',
                'DTMESFINA'       => 'P_DTMESFINA',
                'IDESTOCAVEIS'    => 'P_IDESTOCAVEIS',
                'IDNESTOCAVEIS'   => 'P_IDNESTOCAVEIS',
                'DTPERIFINA'      => 'P_DTPERIFINA',
                'IDDIFERENCA'     => 'P_IDDIFERENCA',
                'DTFINAL'         => 'P_DTFINAL',
                'CDTIPOCUSTOPLAN' => 'P_NRTIPOCUSTOPLAN',
                'CDTIPOCUSTOREAL' => 'P_NRTIPOCUSTOREAL',
                'DTPERIINIC'      => 'P_DTPERIINIC',
                'NMTIPOCUSTOREAL' => 'P_NMTIPOCUSTOREAL',
                'TPDTCUS'         => 'P_TPDTCUS',
                'LISTASERV'       => 'L_SERVICO',
                'NMTIPOCUSTOPLAN' => 'P_NMTIPOCUSTOPLAN',
                'NRCONSPLAN'      => 'P_NRCONSPLAN'
            ),
        ),
        'CUS01102' => 
        array (
        'strategy' => 'BIRT',
        'parameterMapping' => 
            array (
                'NRORG'           => 'P_NRORG',
                'DTINICIAL'       => 'P_DTINICIAL',
                'DTMESINIC'       => 'P_DTMESINIC',
                'DTMESFINA'       => 'P_DTMESFINA',
                'IDESTOCAVEIS'    => 'P_IDESTOCAVEIS',
                'IDNESTOCAVEIS'   => 'P_IDNESTOCAVEIS',
                'DTPERIFINA'      => 'P_DTPERIFINA',
                'IDDIFERENCA'     => 'P_IDDIFERENCA',
                'DTFINAL'         => 'P_DTFINAL',
                'CDTIPOCUSTOPLAN' => 'P_NRTIPOCUSTOPLAN',
                'CDTIPOCUSTOREAL' => 'P_NRTIPOCUSTOREAL',
                'DTPERIINIC'      => 'P_DTPERIINIC',
                'NMTIPOCUSTOREAL' => 'P_NMTIPOCUSTOREAL',
                'TPDTCUS'         => 'P_TPDTCUS',
                'LISTASERV'       => 'L_SERVICO',
                'NMTIPOCUSTOPLAN' => 'P_NMTIPOCUSTOPLAN',
                'NRCONSPLAN'      => 'P_NRCONSPLAN',
                'CDFILIAL'        => 'P_CDFILIAL',
            ),
        ),
    );
    
    //Função para verificar se existe algum grupo sem seviços cadastrados.
    public function validaGrupoServico($nrorg, $cdfilial, $cdoperador, $cdgrupserv, $cdgrupfili = null) {
        $cdgrupserv = explode('_', $cdgrupserv);

        $gruposServico = array();
        $grupoVazio = array();
        $grupoValido = array();
        
            foreach ($cdgrupserv as $cdgrupservs) {
    
                $gruposServico = $this->servicoPorGrupo->retornaServicosPorGrupo($nrorg, $cdfilial, $cdoperador, $cdgrupservs, $cdgrupfili);
                
                if(!$gruposServico){
                    $grupoVazio[] =  $cdgrupservs;
                }else{
                    $grupoValido[] = $cdgrupservs;
                }
            }
       
        if($grupoValido) {
            return implode("_", $grupoValido);
        }else{
            throw new \Exception('Não existe Serviços cadastrado nesse grupo!');
            
        }   
       
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $filiServ = array();
        $dateFinal   = \DateTime::createFromFormat('d/m/Y', $row['DTFINAL']);
        $dateInicial = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);
        
        if(isset($row['LISTAFILI']) && $row['LISTAFILI']) {
            $row['CDFILIAL'] = $row['LISTAFILI'];
        } else {
            $row['CDFILIAL'] = $row['CDFILIAL'];
        }
        
        $cdoperador = $this->environment->getCdOperador();
        $cdfilial   = $row['CDFILIAL'];
        $nrorg      = $this->environment->getNrOrg();
        $cdgrupserv = $row['LISTAGRUPSERV'];
        $listaServ  = $row['LISTASERV'];
        $row['DTPERIINIC'] = $row['DTINICIAL'];
        $row['DTPERIFINA'] = $row['DTFINAL'];
        $validaGrupoComServico = $this->validaGrupoServico($nrorg, $cdfilial, $cdoperador, $cdgrupserv);
        //Atribui somente grupos que tem serviços cadastrados a variavel $cdgrupserv.
        $cdgrupserv = $validaGrupoComServico;

        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));

        if ($row['LISTASERV'] == 'T') {
            if(isset($row['LISTAFILI']) && $row['LISTAFILI']){
                $cdfilial = $row['LISTAFILI'];
            }
            if($row['REPORT_NAME'] != 'CUS01101'){
                $servicos = $this->servicoPorGrupo->retornaServicosPorGrupo($nrorg, $cdfilial, $cdoperador, $cdgrupserv);
                $servicos = explode('_', $servicos);
                $row['LISTASERV'] = [];
                foreach($servicos as $serv) {
                    $row['LISTASERV'][]['CDSERVFILI'] = $serv;
                }
            } else {
                $cdgrupfili = isset($row['CDGRUPFILI']) ?  $row['CDGRUPFILI'] : null;
                $servicos = $this->servicoPorGrupo->retornaServicosPorGrupo($nrorg, $cdfilial, $cdoperador, $cdgrupserv, false, $cdgrupfili);
                $row['LISTASERV'] = $servicos;
            }
        }
        $strLista = '';
        if($row['REPORT_NAME'] == 'CUS01101') { // Este relatório usa a tabela Lista_servico
            $newCodeServ = $this->novoCodigo->getCodeByTableName('LISTA_SERVICO', $row['NRORG']);
            foreach ($row['LISTASERV'] as &$serv) { 
                $serv['NRLISTA'] = $newCodeServ;
                $serv['__is_new'] = true;
                $serv['NMSERVICO'] = isset($serv['NMSERVICO']) ? $serv['NMSERVICO'] : " ";
                $strLista .= $serv['CDSERVFILI']."_";
                if(isset($filiServ[$serv['CDFILIAL']])){
                    $filiServ[$serv['CDFILIAL']] .= '_'.$serv['CDSERVFILI'];
                } else {
                    $filiServ[$serv['CDFILIAL']] = '_'.$serv['CDSERVFILI'];
                }
            }
            $dataSet = new DataSet('lista_servico', $row['LISTASERV']);
            $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
            $rows = $this->crudService->getRowsToSave($newRequest, 'lista_servico');
            $this->crudService->save('lista_servico', $rows);
            $row['LISTASERV'] = $newCodeServ;
            $listaServ = $strLista;
        } else { // Os outros, usam a tabela Lista
            $servicos = [];
            foreach ($row['LISTASERV'] as $serv) { 
                $strLista .= "_".$serv['CDSERVFILI'];
                $servicos[] = $serv['CDSERVFILI'];
            }
            $listaServ = $strLista;
            $row['LISTASERV'] = $this->util->insertLista($servicos);
        }
        
        $row['LISTAFILI'] = $row['CDFILIAL'];

        $codigo = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$row['CDFILIAL'], $row['NRORG']);
        $nrConsPlan = $codigo;
        if($row['REPORT_NAME'] == 'CUS01101'){
            foreach($filiServ as $key => $value){
                $this->dispatchProcedure($row, $key."", $value, $codigo);
            }
        } else {
            $this->dispatchProcedure($row, $cdfilial, $listaServ, $codigo);
        }
        $row['NRCONSPLAN'] = $nrConsPlan;
        $row['NRLISTA']    = $nrConsPlan;
        $row['CDTIPOCUSTOPLAN'] = (int)$row['CDTIPOCUSTOPLAN'];
        $row['CDTIPOCUSTOREAL'] = (int)$row['CDTIPOCUSTOREAL'];
    }
    
    function dispatchProcedure($row, $cdfilial, $listaServ, $codigo){
        $dateInicial = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);
        $dateFinal   = \DateTime::createFromFormat('d/m/Y', $row['DTFINAL']);
        $operador = $this->environment->getCdOperador();    
        $procedureConsumoPlanejado = new StoredProcedure($this->connection, 'CONSUMO_PLANEJADO_PLA');
        $procedureConsumoPlanejado->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT, $cdfilial,    Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',     Param::PARAM_INPUT, $listaServ,          Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTINICIAL'],   Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTFINAL'],     Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, "T",                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, "CFP",               Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, "N",                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                   Param::PARAM_TYPE_INT));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, "N",                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $codigo,             Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, '0',                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, "zzzzzzzzzz",        Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $operador,  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',  Param::PARAM_INPUT, "_".$dateInicial->format('Ymd')."_", Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',    Param::PARAM_INPUT, "_".$dateFinal->format('Ymd')."_",   Param::PARAM_TYPE_STR));
        
        $resultado  = $procedureConsumoPlanejado->execute();
        return $resultado;
    }
    
}