<?php
namespace Teknisa\CUS\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Doctrine\ORM\EntityManager;

class Listas extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $entityManager;
    protected $connection;
    protected $crudService;
    protected $parameterbag;
    protected $newCode;
    protected $dataSourceManager;
    
    public function __construct($crudService, $parameterbag, $environment, $connection, $entityManager, $newCode, $dataSourceManager) {
        $this->crudService       = $crudService;
        $this->parameterbag      = $parameterbag;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->entityManager     = $entityManager;
        $this->newCode           = $newCode;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    //insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista,
            "NRORG" => $this->environment->getNrorgTrab()
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //apaga lista do banco
    public function deleteLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        
        $sql = "DELETE FROM LISTA WHERE NRLISTA =:NRLISTA";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //cria lista de valores recebendo string como parametro
    public function criarListaValores($valores, $tamanho_para_lista=4000){
        if(strlen($valores) <= $tamanho_para_lista){
            return $valores;
        }else{
            $lista = explode('_',$valores);
            $lista = array_unique($lista);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    if($key != ''){
                        $this->insertLista($nrlista, $key);
                    }
                }
                return "_NRLISTA=".$nrlista;
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    public function criarListaValoresMultiplos($ArrayAssociativoDeValores){
        $resultado = array();
        foreach($ArrayAssociativoDeValores as $key => $value){
            $resultado[$key] = $this->criarListaValores($value);
        }
        return $resultado;
    }
    
    //apaga lista de valores recebendo string como parametro
    public function apagarListaValores($valores){
        try{
            if(substr($valores, 0,9) == "_NRLISTA="){
                $nrlista = substr($valores, 9);
                $this->deleteLista($nrlista);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function retornaListaValores(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		$row = $this->criarListaValoresMultiplos($row);
		$response->addDataSet(new DataSet("listaValores", $row));
    }
}